package com.company.testcollection.web.screens.serviceorder;

import com.company.testcollection.core.ServiceOrderService;
import com.company.testcollection.entity.ServiceOrderEmployee;
import com.company.testcollection.entity.ServiceOrderItem;
import com.company.testcollection.entity.ServiceOrderItemDetail;
import com.haulmont.cuba.gui.builders.EditMode;
import com.haulmont.cuba.gui.components.Button;
import com.haulmont.cuba.gui.components.GroupTable;
import com.haulmont.cuba.gui.components.Table;
import com.haulmont.cuba.gui.components.data.TableItems;
import com.haulmont.cuba.gui.components.data.table.ContainerGroupTableItems;
import com.haulmont.cuba.gui.model.CollectionContainer;
import com.haulmont.cuba.gui.model.CollectionPropertyContainer;
import com.haulmont.cuba.gui.model.DataContext;
import com.haulmont.cuba.gui.screen.*;
import com.company.testcollection.entity.ServiceOrder;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.List;

@UiController("testcollection_ServiceOrder.edit")
@UiDescriptor("service-order-edit.xml")
@EditedEntityContainer("serviceOrderDc")
@LoadDataBeforeShow
public class ServiceOrderEdit extends StandardEditor<ServiceOrder> {

    @Inject
    private DataContext dataContext;
    @Inject
    private CollectionPropertyContainer<ServiceOrderItem> serviceOrderItemsDc;
    @Inject
    private Table<ServiceOrderItem> serviceOrdersTable;
    @Inject
    private ServiceOrderService serviceOrderService;
    @Inject
    private CollectionContainer<ServiceOrderItemDetail> serviceOrderItemDetailContainer;
    @Inject
    private GroupTable<ServiceOrderItemDetail> serviceOrdersTableDetail;

    public void initInserted() {
        List<ServiceOrderItem> serviceOrderItems = generateServiceOrderItemList();
        serviceOrderItemsDc.setItems(serviceOrderItems);

        this.getEditedEntity().setServiceOrderItems(serviceOrderItems);
        this.getEditedEntity().setOrderState(1);
    }

    public void initEdited() {
        serviceOrdersTable.setEditable(false);
        loadServiceOrderDetailToDetailContainer(this.getEditedEntity());
        this.getEditedEntity().setOrderState(2);
    }

    private List<ServiceOrderItem> generateServiceOrderItemList() {
        // в реальности генерируется запросом к базе
        List<ServiceOrderItem> serviceOrderItems = new ArrayList<>();
        for (int i = 0; i < 3; i++) {
            ServiceOrderItem serviceOrderItem = dataContext.create(ServiceOrderItem.class);
            serviceOrderItem.setOrderQuantity(0.0);
            serviceOrderItem.setName("ServiceOrderItem_" + i);
            serviceOrderItem.setServiceOrder(this.getEditedEntity());
            serviceOrderItems.add(serviceOrderItem);
        }
        return serviceOrderItems;
    }

    private void loadServiceOrderDetailToDetailContainer(ServiceOrder serviceOrder) {
        List<ServiceOrderItemDetail> serviceOrderItemDetails = serviceOrderService.getServiceOrderItemDetailListByServiceOrder(serviceOrder);

        if (serviceOrderItemDetails.size() == 0) {
            for (int i = 0; i < serviceOrder.getServiceOrderItems().size(); i++) {
                ServiceOrderItem serviceOrderItem = serviceOrder.getServiceOrderItems().get(i);

                for (int j = 0; j <= i; j++) {
                    ServiceOrderItemDetail newServiceOrderItemDetail = dataContext.create(ServiceOrderItemDetail.class);
                    newServiceOrderItemDetail.setServiceOrderItem(serviceOrderItem);
                    newServiceOrderItemDetail.setDuration(0.0);

                    ServiceOrderEmployee serviceOrderEmployee = dataContext.create(ServiceOrderEmployee.class);
                    serviceOrderEmployee.setName("serviceOrderItem_" + i + "_Имя_" + j);
                    newServiceOrderItemDetail.setServiceOrderEmployee(serviceOrderEmployee);

                    serviceOrderItemDetails.add(newServiceOrderItemDetail);
                }

            }
        }
        serviceOrderItemDetailContainer.setItems(serviceOrderItemDetails);
        TableItems<ServiceOrderItemDetail> tableItems = new ContainerGroupTableItems<>(serviceOrderItemDetailContainer);
        serviceOrdersTableDetail.setItems(tableItems);
        serviceOrdersTableDetail.groupByColumns("serviceOrderItem.name");
        serviceOrdersTableDetail.expandAll();
    }

    @Subscribe("forTest")
    public void onForTestClick(Button.ClickEvent event) {
        List<ServiceOrderItemDetail> serviceOrderItemDetails = serviceOrderItemDetailContainer.getMutableItems();

        ServiceOrderItem serviceOrderItem = serviceOrderItemDetails.get(3).getServiceOrderItem();
        serviceOrderItemDetails.remove(3);

        ServiceOrderItemDetail newServiceOrderItemDetail = dataContext.create(ServiceOrderItemDetail.class);
        newServiceOrderItemDetail.setServiceOrderItem(serviceOrderItem);
        newServiceOrderItemDetail.setDuration(0.0);

        ServiceOrderEmployee serviceOrderEmployee = dataContext.create(ServiceOrderEmployee.class);
        serviceOrderEmployee.setName("serviceOrderItem_2_Имя_3");
        newServiceOrderItemDetail.setServiceOrderEmployee(serviceOrderEmployee);

        serviceOrderItemDetails.add(newServiceOrderItemDetail);
    }
}